%% THIS IS AN M-FILE THAT DEMONSTRATES MATLAB(R)'S CAPABILITY IN IMAGE AQUISITION AND ANALYSIS
%% 0. CLOSE ALL FIGURES AND CLEAR VARIABLES BEFORE YOU START
close all; clear all;

%% 1. READ THE IMAGE FILE
FileName='D:\MATLAB\R2011b\toolbox\images\imdemos\candy';
candy=imread(FileName,'jpg');

%% 2. CONVERSION FROM TRUECOLOR TO GRAYSCALE IMAGE
% I=rgb2gray(candy)converts the truecolor image candy to the grayscale
% intensity image, I. It eliminates the hue and saturation information
% (chromatic) while retains the luminance (intensity).
I=rgb2gray(candy);
figure(1); imshow(I);

%% 3. 1st MORPHOLOGICAL IMAGE TREATMENT: CLOSE OPERATION (OBJECT INTERCONNECTION)
% imclose performs morphological closing on the grayscale or binary image
% I, returning the closed image, bckgrnd. The structuring element, strel,
% must be a single structuring element object, as opposed to an array of
% objects. Create a disk-shaped structuring element. Use a disk structuring
% element to preserve the circular nature of the object. Specify a radius
% of 12 pixels so that the largest gap gets filled. Perform a morphological
% close operation on the image. Note: 12 pixel radius is smaller than the
% radius of the smallest Mars pellet found in the picture. The
% morphological close operation (imclose)is a dilation (see, imdilate for
% help)followed by an erosion (see, imerode for help), using the same
% structuring element for both operations.
bckgrnd=imclose(I,strel('disk',12));
figure(2); imshow(bckgrnd);

%% 4. 2nd MORPHOLOGICAL TREATMENT: REMOVAL OF BACKGROUND LUMINANCE
% I2 = imsubtract(bckgrnd,I) subtracts each element in array I from the 
% corresponding element in array bckgrnd and returns the difference in 
% the corresponding element of the output array I2
I2=imsubtract(bckgrnd,I);% Equivalent to I2=bckgrnd-I;
figure(3);imshow(I2,[]);

%% 5. SETTING A THRESHOLD FOR LIGHT INTENSITY (LUMINANCE)
% level= graythresh(I2) computes a global threshold (level between 0
% and 1) that can be used to convert an intensity image to a binary image
% with im2bw.
level=graythresh(I2);

%% 6. CONVERSION OF GRAY IMAGE INTO BINARY IMAGE WHILE UTILIZING THE THRESHOLD
% BW = im2bw(I2, graythresh(I2)) converts the grayscale image I2 to
% a binary image, BW. The output image BW replaces all pixels in the input
% image with luminance greater than level with the value 1 (white) and
% replaces all other pixels with the value 0 (black).Notice that I, I2, and
% bckgrnd are gray-scale arrays of uint8 type, which have values that range
% from 0 to 255.  On the other hand, BW is logical that has 0 or 1 pixel
% value.
BW=im2bw(I2,level);

%% 7. 3RD MORPHOLOGICAL TREATMENT: REMOVAL OF ARTIFACTS (SMALL OBJECTS LESS THAN 20 PIXELS)
% BW2=bwareaopen(BW,20) Removes from a binary image, BW, all connected
% components (objects) that are fewer than 20 pixels(object area), producing
% another binary image, BW2. The default connectivity is 8 for two
% dimensions.8 means 8-connected neighborhood.
BW2=bwareaopen(BW,20);
figure(4); imshow(BW2);

%% 8. 4TH MORPHOLOGICAL TREATMENT: REMOVAL OF BORDER ARTIFACTS (CONNECTED TO THE IMAGE BORDER)
% BW3 = imclearborder(BW2) suppresses structures that are both lighter than
% their surroundings and are connected to the image border. (In other
% words, use this function to clear the image border.) BW2 can be a
% grayscale or binary image. The output image, BW3, is grayscale or binary,
% respectively. The default connectivity is 8 for two dimensions.Since we
% do not have border artifacts, the image shown in Figure (4) and that in
% Figure (5) are essentially identical.
BW3=imclearborder(BW2);
figure(5); imshow(BW3);

%% 9. 5TH MORPHOLOGICAL TREATMENT: FILLING IN THE GAPS WITHIN OBJECTS
% fill=imfill(BW3,'holes') fills holes in the binary image BW3. A hole is 
% a set of background pixels that cannot be reached by filling in the backg-
% round from the edge of the image.
fill=imfill(BW3,'holes');
figure(6); imshow(fill);
%% 10. FIND CONNECTED COMPONENTS IN A BINARY IMAGE
%ConComp = bwconncomp(fill) returns the struct, ConComp, of the connected
%components that are found in THE FINAL, FINISHED BINARY IMAGE, fill.
%ConComp is a struct with four fields.  One important field is number of
%objects, NumObjects.  A struct is a structure array, which has fields.
%The most common way to access the data in a struct is by specifying the
%name of the field that we want to reference.  Use the name of the struct
%followed by the dot . and immediately by the name of the field.
[ConComp]=bwconncomp(fill);

%% 11. DETERMINE OBJECT'S GEOMETRICAL AND SHPAE PROPERTIES
% STATS = regionprops(ConComp, properties) measures a set of properties for
% each connected component (object) in ConComp, which is a structure
% returned by bwconncomp. Examples of properties are: Area, BoundingBox,
% Centroid, Eccentricity, EquivDiameter, Extent, MajorAxisLength,
% MinorAxisLength, and Perimeter. Please, see regionprops using Product Help.
% Here, there properties were chosen which are: Eccentricity, Area, and
% BoundingBox.
Stats=regionprops(ConComp,'Eccentricity','Area','BoundingBox');

%% 12. REDEFINING VARIABLES FOR SIMPLICITY
% STATS is a structure array with length equal to the number of objects as
% ConComp.NumObjects. The fields of the structure array denote different
% properties for each region, as specified by properties.
% Area=[Stats.Area] extracts the Stats field, that is, Area into a
% separate vector, called Area. Area has one value for each object
% (pellet). Eccentricity=[Stats.Eccentricity] facilitates the usage of the
% property: eccentricity. Eccentricity has one unique value for each object.
Area=[Stats.Area];
Eccentricity=[Stats.Eccentricity];

%% 13. DEFINE MINIMUM AREA BELOW WHICH THE PELLET FAILS THE QUALITY CONTROL TEST!
% A restriction to accept a candy pellet as undamaged is to have its actual
% area been greater than a minimum area named: MinArea.  MinArea, a scalar
% quantity, is defined as the arithmetic mean minus one fourth times the
% standard deviation of pellets (objects) area.  Keep in mind that Area
% is 18 row vector, where 8 is the number of found objects using
% bwconncomp command in Step 10.
MinArea=mean(Area)-0.25*std(Area);

%% 14. USE MINAREA & ECCENTRICTY VALUES TO IDENTIFY BROKEN OBJECTS (CANDY PELLETS)
% If the area of the pellet (object) is below the threshold value, MinArea,
% and if the eccentricity is above 0.5, then the object is said to have
% defects . The eccentricity is the ratio of the distance between the foci
% of the ellipse and its major axis length. The value is between 0 and 1.
% Zero means a full circle and one a line segment, in 2-D shapes.
% IndexOfDefect is the index of the matrix element returned by find
% statement once the condition is met or fulfilled.
% StatDefect=Stats(IndexOfDefect) picks up only the defected objects.
% StatDefect is also a struct, similar to Stats; however, it contains only
% objects that fail the quality control test. It is worth-mentioning here
% that for each entry found in either struct, it has three associated
% attributes which are: Area, BoundingBox, and Eccentricity.
IndexOfDefect=find(Area<MinArea & Eccentricity> 0.5);
StatDefect=Stats(IndexOfDefect);

%% 15. LABELING OF BROKEN OBJECTS
% Labeling, in the form of drawing a bounding box, is now carried out on
% the original, unmodified image. hold on retains the current plot and
% certain axes properties so that subsequent graphing commands add to the
% existing graph. The for loop is carried out all over defected objects;
% where a recangle is drawn around each defected object. Keep in mind that
% a defected object is the object that has area that is below the threshold
% value, MinArea, and its eccentricity value lies above 0.5.
figure(7); imshow(candy);
hold on;
for index=1:length(StatDefect)
    h=rectangle('Position',StatDefect(index).BoundingBox,'LineWidth',3);
    set(h,'EdgeColor',[0.75 0 0]);
end
hold off;
